#include <math.h>
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>	/* Get standard string definations. */
#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>

#include <X11/Xaw/Box.h>	
#include <X11/Xaw/Command.h>	
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Label.h>	
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Cardinals.h>	

#include "pbitdraw.h"  		/* header file for tile widget */
#define CANVASCOLOR 1
#define MATRIX_DIM 3		/* box dimension data*/
#define VERTICES 8
#define EDGES 12
#define FACES 6
#define FACE_VERS 4		/* number of vertices on a face */
#define MAXSTRING 66
#define OFFSCREEN -10
#define MISSED_BOX -10000	/* point not on world box */
#define UNIT_AREA 5

#define HALFSIZE 80		/* Half the width/height/len of the
					"unit" cube */
#define LEFT 1			/* flags for left and right matrix
					multiplication */
#define RIGHT -1

#define Z_PLANE 400 		/* projection plane */ 
#define INVISIBLE 10000         /* flag denoting a backface */
#define ZNEAR 116               /* near plane */
#define ZFAR  687		/* far plane */
#define XWMAX 280  		/* world window boundaries */    
#define XWMIN -280 		/* sizes based on allowing a maximum size
				   box free rotation */
#define YWMAX 280  
#define YWMIN -280
#define XVMAX 470   		/* view window boundaries */
#define XVMIN 0
#define YVMAX 400 
#define YVMIN 0
				/* constants used in view transformation*/
#define MAX(x,y)  x > y ? x:y
#define MIN(x,y)  x < y ? x:y
#define SX (XVMAX - XVMIN)/(XWMAX - XWMIN)
#define SY (YVMAX - YVMIN)/(YWMAX - YWMIN)

#define COS .9961947  /* cos 5 degrees */
#define SIN .0871557  /* sin 5 degrees */ 

typedef struct {		/* Point in 3D space */
	     	  double x;
		  double y;
		  double z;
} Vector;

typedef Vector VertexTable[VERTICES];

typedef struct {                 /* indices to two vertices */
		int v1;
		int v2;
}	Edge;

typedef Edge EdgeTable[EDGES];

typedef int PolygonTable[FACES][FACE_VERS];  /* a 6x4 array */

typedef struct {		/* Box: vertices, edges, faces */
		VertexTable	vertices;
		EdgeTable	edges;
		PolygonTable    faces;  
}	Box;

typedef struct {		/* screen coordinates of a point */
	double x;
	double y;
}	OrderedPair;

typedef struct {		/* pixel coordinates of a point */
	int x;
	int y;
}	PixelPair;


typedef struct {		/* normal to a face and a point*/ 
	Vector normal;
        Vector pt;
} PlaneEquation;

	
typedef double Matrix[MATRIX_DIM][MATRIX_DIM]; 	/* matrix */


			 	/* data structure containing important 
			      	   screen and box information   */
typedef struct {
	Widget  target;		/* canvas */
	Widget  message;	/* help and error message window */
	Widget  sliderlabel;	/* label showing the current scale */
        Widget  slider0;	/* three scale sliders */
        Widget  slider1;
	Widget  slider2;
	Widget  tilewin;	/* the window showing tiles */
	Display *w_display;	/* display, window of canvas */
	Window  window;
	GC	gc;		/* graphics context */
	int     currentslider;	/* flag identifies current slider*/
 	int 	currentrotate;	/* flag identifies which axis */
	int 	rotatepos;	/* mark position of rotate slider*/
	double  x_scale;	/* X,Y,Z scale values */
        double  y_scale;
        double  z_scale;
        double  xscale_copy;	/* copy of scale values */
        double  yscale_copy;
        double  zscale_copy;
	int 	x_translate;	/* translation values */
	int	y_translate;
	int 	z_translate;
}	canvas_data;

/*
 *  	Global variables
 */
Box originalbox;        /* This is a permanent world model. */
Box box;  		/* This is a permanent world model */ 
Matrix  rotation; 	/* Rotation matrix */
Matrix  back_rotation; 	/* Inverse rotation matrix */
PixelPair  screen_region[FACES][VERTICES];  /* Hold vertices of
				visible faces on the screen */

/* holds plane equations for pre and post projected faces */
PlaneEquation proj_box[FACES], world_box[FACES]; 

