/*********************************************************************
MODULE:         rotate.c

PROGRAMMERS:    John White and John White

DESCRIPTION:    This modules contains functions for rotation,
                translation, and the put_message() function
                which writes information to the help window.
*******************************************************************/
#include "p2.h"

extern render_box();
extern init_matrix();
extern fix_rotation_matrix();
extern proj(), in_range(), transform(), copy_box();
extern back_face();

/********************************************************************
FUCNTION:	rev_x_rotate
PURPOSE:        This is a callback function which rotates the box in
		 the opposite x direction about its center 	
PARAMETERS:     w - The x_rotate widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
rev_x_rotate(data,direction)
  canvas_data *data;
  int direction;
{
void put_message();
Matrix new_matrix;
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

    init_matrix(new_matrix);		/* create empty matrix */

    new_matrix[0][0] =  1;		/* set key values in matrix */
    new_matrix[1][1] =  COS; 
    new_matrix[1][2] = -SIN*direction; 
    new_matrix[2][1] =  SIN*direction; 
    new_matrix[2][2] =  COS; 
    new_matrix[3][3] =  1;

    fix_rotation_matrix(new_matrix,rotation,RIGHT);   /* adjust global rotation matrix*/

transform(mesh, data);
proj(mesh, data);

init_matrix(new_matrix);

new_matrix[0][0] =  1;	/* inverse matrix */
new_matrix[1][1] =  COS; 
new_matrix[1][2] =  SIN*direction; 
new_matrix[2][1] = -SIN*direction; 
new_matrix[2][2] =  COS; 
new_matrix[3][3] =  1;

if( in_range(mesh,data))			/* boundary check */
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");

					/* update inverse matrix*/
             fix_rotation_matrix(new_matrix,back_rotation,LEFT);
	}
/* Out of bounds, so restore global rotation matrix without updating
	back_rotation, the global inverse matrix		*/ 
else
             fix_rotation_matrix(new_matrix,rotation,RIGHT);
         
free(mesh);
}

/********************************************************************
FUCNTION:	rev_y_rotate
PURPOSE:        This is a callback function which rotates the box in
		 the opposite y direction about its center 	
PARAMETERS:     w - The y_rotate widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
rev_y_rotate(data,direction)
  canvas_data *data;
  int direction;
{
void put_message();
Matrix new_matrix;
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

    init_matrix(new_matrix);

	new_matrix[0][0] =  COS; 
	new_matrix[1][1] =  1; 
	new_matrix[2][0] = -SIN*direction; 
	new_matrix[0][2] =  SIN*direction; 
	new_matrix[2][2] =  COS; 
	new_matrix[3][3] =  1;

    fix_rotation_matrix(new_matrix,rotation,RIGHT);

transform(mesh, data);
proj(mesh, data);

init_matrix(new_matrix);

new_matrix[0][0] =  COS; 
new_matrix[1][1] =  1; 
new_matrix[2][0] =  SIN*direction; 
new_matrix[0][2] = -SIN*direction; 
new_matrix[2][2] =  COS; 
new_matrix[3][3] =  1;

if( in_range(mesh, data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");

				/* update global inverse matrix*/
             fix_rotation_matrix(new_matrix,back_rotation,LEFT);
         }
/* Out of bounds, so restore global rotation matrix without updating
	back_rotation, the global inverse matrix		*/ 
else
             fix_rotation_matrix(new_matrix,rotation,RIGHT);
         
free(mesh);
}

/********************************************************************
FUCNTION:	rev_z_rotate
PURPOSE:        This is a callback function which rotates the box in
		 the x direction about its center 	
PARAMETERS:     w - The z_rotate widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
rev_z_rotate(data,direction)
  canvas_data *data;
  int direction;
{
void put_message();
Matrix new_matrix;
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

    init_matrix(new_matrix);

	new_matrix[0][0] =  COS; 
	new_matrix[1][0] =  SIN*direction; 
	new_matrix[0][1] = -SIN*direction; 
	new_matrix[1][1] =  COS; 
	new_matrix[2][2] =  1; 
	new_matrix[3][3] =  1;


    fix_rotation_matrix(new_matrix,rotation,RIGHT);

transform(mesh, data);
proj(mesh, data);

init_matrix(new_matrix);

new_matrix[0][0] =  COS; 
new_matrix[1][0] = -SIN*direction; 
new_matrix[0][1] =  SIN*direction; 
new_matrix[1][1] =  COS; 
new_matrix[2][2] =  1; 
new_matrix[3][3] =  1;

if( in_range(mesh, data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");	

				/*update the global inverse matrix*/
             fix_rotation_matrix(new_matrix,back_rotation,LEFT);
         }
/* Out of bounds, so restore global rotation matrix without updating
	back_rotation, the global inverse matrix		*/ 
else
             fix_rotation_matrix(new_matrix,rotation,RIGHT);
         
free(mesh);
}
/********************************************************************
FUCNTION:       put_message	
PURPOSE:        Writes a message in the message window
PARAMETERS:	call_data - holds important screen and box information
		message   - A string that is to be written
*******************************************************************/
void put_message(call_data,message)
  canvas_data *call_data;
  char *message;
{
char  str[MAXSTRING]; 
int i;
Arg	args[1];

sprintf(str,message);
	XtSetArg(args[0],XtNlabel,str);
	XtSetValues(call_data->message,args,ONE);
}

/********************************************************************
FUCNTION: 	zoom_in_move	
PURPOSE:        This is a callback function which moves the box in
		 the positive z direction
PARAMETERS:     w - The zoom_in widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void zoom_in_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->z_translate -= 5;		/* adjust z offset */

transform(mesh, data);
proj(mesh, data);
if( in_range(mesh, data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");
	}
else			/* out of bounds, so z offset */
	data->z_translate += 5;
free(mesh);
}

/********************************************************************
FUCNTION: 	zoom_out_move	
PURPOSE:        This is a callback function which moves the box in
		 the negative z direction
PARAMETERS:     w - The zoom_out_move widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void zoom_out_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->z_translate += 5;

transform(mesh, data);
proj(mesh, data);
if( in_range(mesh,data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");
	}
else
	data->z_translate -= 5;
free(mesh);
}

/********************************************************************
FUCNTION:       up_move	
PURPOSE:        This is a callback function which moves the box in
		 the negative y direction
PARAMETERS:     w - The up_move widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void up_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->y_translate-=5;

transform(mesh, data);
proj(mesh, data);
if( in_range(mesh, data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");
	}
else
	data->y_translate+=5;
free(mesh);
}

/********************************************************************
FUCNTION:       down_move	
PURPOSE:        This is a callback function which moves the box in
		 the positive y direction
PARAMETERS:     w - The down_move widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void down_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->y_translate+=5;

transform(mesh, data);
proj(mesh, data);
back_face(mesh);
if( in_range(mesh,data))
	{
		XClearWindow(data->w_display, data->window);
		render_box(mesh,data);
		put_message(data,"");
	}
else
	data->y_translate-=5;
free(mesh);
}

/********************************************************************
FUCNTION:       right_move	
PURPOSE:        This is a callback function which moves the box in
		 the positive x direction
PARAMETERS:     w - The right_move widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void right_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->x_translate+=5;

transform(mesh, data);
proj(mesh, data);
if( in_range(mesh,data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");
	}
else
	data->x_translate-=5;
free(mesh);
}

/********************************************************************
FUCNTION:       left_move	
PURPOSE:        This is a callback function which moves the box in
		 the negative x direction 
PARAMETERS:     w - The left_move widget which triggered this callback
		data - holds important screen and box information
		call_data - not used, syntactic necessity for callback
*******************************************************************/
void left_move(w,data,call_data)
  Widget w;
  canvas_data *data;
  XtPointer call_data;
{
Box *mesh;

mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

data->x_translate-=5;

transform(mesh, data);
proj(mesh, data);
if( in_range(mesh,data))
	{
		XClearWindow(data->w_display, data->window);
		back_face(mesh);
		render_box(mesh,data);
		put_message(data,"");
	}
else
	data->x_translate+=5;
free(mesh);

}

