/*********************************************************************
MODULE:         texture.c 

PROGRAMMERS:    John White and John White

DESCRIPTION:    This modules contains functions for tiling   
		the faces of the box.
*******************************************************************/

#include "p2.h"
extern void put_message();
/***************************************************************
FUNCTION:	init_screen_region
PURPOSE:	initialize the values of the global screen_region
			variable
PARAMETERS:	none
***************************************************************/


void init_screen_region()
{
int i,j;

for(i=0;i<FACES;i++)
	for(j=0;j<VERTICES;j++)
		{
			screen_region[i][j].x = OFFSCREEN;
			screen_region[i][j].y = OFFSCREEN;
		}
}
			
/***************************************************************
FUNCTION:	order_screen_region
PURPOSE:        This finds the max and min x and y values for
		each visible face.  min is stored in
		screen_region[face][0] and max is stored in
		screen_region[face][1].	
PARAMETERS:	none
***************************************************************/
void order_screen_region()
{
int i,j;
int max_x=0,max_y=0,min_x=10000,min_y=10000;

for(i=0;i<FACES;i++)
	if (screen_region[i][0].x != OFFSCREEN)
	{
	   for(j=0;j<VERTICES;j++)  {
		max_x = MAX(max_x,screen_region[i][j].x);
		max_y = MAX(max_y,screen_region[i][j].y);
		min_x = MIN(min_x,screen_region[i][j].x);
		min_y = MIN(min_y,screen_region[i][j].y);
	   }
	screen_region[i][0].x = min_x;
	screen_region[i][0].y = min_y;
	screen_region[i][1].x = max_x;
	screen_region[i][1].y = max_y;

	max_x = max_y = 0;
	min_x = min_y = 10000;
	}
}

/***************************************************************
FUNCTION:       ToWorldCoords	
PURPOSE:        This is the inverse function to ToScreenCoords 
		It converts from screen coordinates to
			projected world coordinates
PARAMETERS:     pt - 	An ordered pair representing a screen
			point
**************************************************************/
OrderedPair	ToWorldCoords(pt)
  OrderedPair pt;
{
OrderedPair result;

result.x = (pt.x - XVMIN)/((double) SX)  +  XWMIN;
result.y = (pt.y - YVMIN)/((double) SY)  +  YWMIN;

return result;
}
/***************************************************************
FUNCTION:       unproj 
PURPOSE:        This is a partial inverse function of proj 
		It converts from projected screen coordinates to
			pre-projected world coordinates
PARAMETERS:     pt - 	A vector containing the coordinates of
			a projected world point
**************************************************************/
Vector unproj(pt)
Vector pt;
{
Vector result;

result.z =  (Z_PLANE)/(1 - pt.z); 
result.x  = result.z * pt.x / Z_PLANE;
result.y  = result.z * pt.y / Z_PLANE;

return result;
}

/***************************************************************
FUNCTION:       get_z 
PURPOSE:        Find the z coordinate of a vector. Based on the 
	        plane equation defined by a point and a normal.	
PARAMETERS:     norm -  A normal vector to a plane	
		tail - The vertex at the base of the normal 
		tip  - An ordered pair containing the x and y
			components of the vector with unknown z
**************************************************************/
double   get_z (norm, tail, tip)
Vector norm, tail;
OrderedPair  tip;
{
return  ((norm.x*(tail.x - tip.x) + norm.y*(tail.y - tip.y))
	/	norm.z) + tail.z; 
}
/***************************************************************
FUNCTION:       untranslate
PURPOSE:        This is the inverse function of translate 
PARAMETERS:     pt - 	A vector in 3-space 
		data -  Holds translation information
**************************************************************/
Vector	untranslate(pt, data)
  Vector  pt;
  canvas_data  *data;
{
Vector result;

	result.x = pt.x - data->x_translate;
	result.y = pt.y - data->y_translate;
	result.z = pt.z - data->z_translate;

return result;
}

/***************************************************************
FUNCTION:       unrotate
PURPOSE:        This is the inverse function of rotate 
		Uses global inverse matrix  back_rotation
PARAMETERS:     pt - 	A vector in 3-space 
**************************************************************/
Vector	unrotate(pt)
   Vector pt;
{
int i;
double result[MATRIX_DIM];
Vector answer;

for(i=0;i<MATRIX_DIM;i++)  
    result[i] = (pt.x * back_rotation[i][0]) +
		(pt.y * back_rotation[i][1]) +
		(pt.z * back_rotation[i][2]);

answer.x = result[0];
answer.y = result[1];
answer.z = result[2];

return answer;
}

/***************************************************************
FUNCTION:       get_point 
PURPOSE:        Given a screen point, this funtion returns
		the value of the tile area on the corresponding
		world box, or it returns an error if the point
		is outside the world box. 
PARAMETERS:     x - real valued screen x coordinate (subpixel)
		y - real valued screen y coordinate (subpixel)
		face - an index noting the current face
		data - holds important data structures and screen
			information 
**************************************************************/
int get_point (x, y, face, data)
double x,y;
int face;
canvas_data *data;
{
OrderedPair  op;
Vector v;
double i,j,i_scale,j_scale;

op.x = x;
op.y = y;

op = ToWorldCoords(op);

v.z = get_z(proj_box[face].normal, proj_box[face].pt, op);
v.x = op.x;
v.y = op.y;

v = unproj(v);
v = untranslate(v,data);
v = unrotate(v);

switch (face) {
	
	case 0:	i = -v.x;
		j =  v.y;
		i_scale = data->x_scale;
		j_scale = data->y_scale;
		break;

	case 1:	i =  v.x;
		j =  v.z;
		i_scale = data->x_scale;
		j_scale = data->z_scale;
		break;

	case 2:	i =  v.z;
		j =  v.y;
		i_scale = data->z_scale;
		j_scale = data->y_scale;
		break;

	case 3:	i =  v.x;
		j =  v.z;
		i_scale = data->x_scale;
		j_scale = data->z_scale;
		break;

	case 4:	i = -v.z;
		j =  v.y;
		i_scale = data->z_scale;
		j_scale = data->y_scale;
		break;

	case 5:	i =  v.x;
		j =  v.y;
		i_scale = data->x_scale;
		j_scale = data->y_scale;
		break;

	default: exit(1);
		 break;
    }

    if (i < (i_scale * HALFSIZE) && i > (i_scale * -HALFSIZE) && 
        j < (j_scale * HALFSIZE) && j > (j_scale * -HALFSIZE))

	return tile[face][(int) ((i/UNIT_AREA)+ (HALFSIZE*i_scale))% TILE_DIM] 
			 [(int) ((j/UNIT_AREA)+ (HALFSIZE*j_scale))% TILE_DIM];
    else
	return MISSED_BOX;
}

/***************************************************************
FUNCTION:       texture 
PURPOSE:        Covered every visible face of the projected
		box with its corresponding texture map. 
		This is a callback to the texture button
PARAMETERS:     w - widget of callback (texture button) 
		in_data - holds important data structures
				and screen information
		call_data - not used, X requirements for callbacks
**************************************************************/
void texture(w, in_data, call_data)
Widget w;
canvas_data *in_data;
XtPointer call_data;
{
int f,i,j,i_lim,j_lim;
int shade;
Box *mesh;

/* first clear for texture map (may already be textured */
mesh = (Box *) malloc (sizeof(Box));
copy_box(mesh);

put_message(in_data,"working ...");
transform(mesh, in_data);
proj(mesh, in_data);
XClearWindow(in_data->w_display, in_data->window);
back_face(mesh);
render_box(mesh,in_data);

put_message(in_data,"working ...");

for(f=0; f<FACES; f++)
   if(screen_region[f][0].x != OFFSCREEN)   /* face is visible */
      {
	/* set limits for point search */
	i = screen_region[f][0].x;
	i_lim = screen_region[f][1].x;
	j_lim = screen_region[f][1].y;
	
        XSetFunction(in_data->w_display,in_data->gc,GXcopy);
	for(; i <= i_lim; i++) 
	   for(j = screen_region[f][0].y,shade=0; j <= j_lim; j++)  {
             shade += get_point((double)(i + .25), (double)(j + .25), f, in_data);
             if(shade >= 0) {
	        shade += get_point((double)(i + .25), (double)(j + .75), f, in_data);
		if(shade >= 0) {
		shade += get_point((double)(i + .75), (double)(j + .25), f, in_data);
		   if(shade >= 0) {
		      shade += get_point((double)(i + .75), (double)(j + .75), f, in_data);
		    switch (shade) {

			case 0:	XSetForeground(in_data->w_display,in_data->gc,0);
				     break;

			case 1: XSetForeground(in_data->w_display,in_data->gc,91);
				     break;

			case 2: XSetForeground(in_data->w_display,in_data->gc,52);
				     break;

			case 3:  XSetForeground(in_data->w_display,in_data->gc,128);
				     break;

			case 4:  XSetForeground(in_data->w_display,in_data->gc,4);
				     break;

			default: XSetForeground(in_data->w_display,in_data->gc,0);
					break;
                            }


			XDrawPoint(in_data->w_display, in_data->window,
				in_data->gc, i, j);
       		}
              }
	    }
		/* reset shade value */
			shade = 0;
         }
      }
put_message(in_data,"");
XSetForeground(in_data->w_display,in_data->gc,1);
}

 /***************************************************************
   FUNCTION:       orientation
   PURPOSE:        Show number corresponding to visible faces
   PARAMETERS:     w - widget of callback (texture button)
   in_data - holds important data structures
   and screen information
   call_data - not used, X requirements for callbacks
   **************************************************************/
   void orientation(w, in_data, call_data)
   Widget w;
   canvas_data *in_data;
   XtPointer call_data;
   {
   Box *mesh;
   int f;
   char str[2];
   OrderedPair pt1;
   
  /* first clear for texture map (may already be textured */
   mesh = (Box *) malloc (sizeof(Box));
   copy_box(mesh);
   
  transform(mesh, in_data);
   proj(mesh, in_data);
   XClearWindow(in_data->w_display, in_data->window);
   back_face(mesh);
   render_box(mesh,in_data);
   
  XSetFunction(in_data->w_display, in_data->gc, GXcopy);
   XSetForeground(in_data->w_display, in_data->gc, 128);
   
   
   for(f=0; f<FACES; f++)
   if( mesh->faces[f][0] != INVISIBLE )
   {
   	if (f == 4)
                     sprintf(str,"%d",(f+2));
             else
                     sprintf(str,"%d", (f+2)%MAX_TEXTURE_MAPS);
   
             XDrawString(in_data->w_display, in_data->window, in_data->gc,
             (screen_region[f][0].x + screen_region[f][1].x)/2 ,
             (screen_region[f][0].y + screen_region[f][1].y)/2 ,
                      str, strlen(str));

     }
     XSetForeground(in_data->w_display, in_data->gc, 1);
 }


